@@ -2,6 +2,8 @@
  *      lxrandr.c - Easy-to-use XRandR GUI frontend for LXDE project
  *
  *      Copyright (C) 2008 Hong Jen Yee(PCMan) <pcman.tw@gmail.com>
+ *      Copyright (C) 2011 Julien Lavergne <julien.lavergne@gmail.com>
+ *      Copyright (C) 2014 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -29,6 +31,16 @@
 #include <locale.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
+
+typedef enum
+{
+    PLACEMENT_DEFAULT,
+    PLACEMENT_RIGHT,
+    PLACEMENT_ABOVE,
+    PLACEMENT_LEFT,
+    PLACEMENT_BELOW
+} MonitorPlacement;
 
 typedef struct _Monitor
 {
@@ -38,12 +50,18 @@ typedef struct _Monitor
     short active_rate;
     short pref_mode;
     short pref_rate;
+    short try_mode;
+    short try_rate;
+    MonitorPlacement placement;
+    MonitorPlacement try_placement;
 
     GtkCheckButton* enable;
 #if GTK_CHECK_VERSION(2, 24, 0)
+    GtkComboBoxText* pos_combo;
     GtkComboBoxText* res_combo;
     GtkComboBoxText* rate_combo;
 #else
+    GtkComboBox* pos_combo;
     GtkComboBox* res_combo;
     GtkComboBox* rate_combo;
 #endif
@@ -68,9 +86,9 @@ static const char* get_human_readable_name( Monitor* m )
     if( m == LVDS )
         return _("Laptop LCD Monitor");
     else if( g_str_has_prefix( m->name, "VGA" ) || g_str_has_prefix( m->name, "Analog" ) )
-        return _( LVDS ? "External VGA Monitor" : "VGA Monitor");
+        return LVDS ? _("External VGA Monitor") : _("VGA Monitor");
     else if( g_str_has_prefix( m->name, "DVI" ) || g_str_has_prefix(m->name, "TMDS") || g_str_has_prefix(m->name, "Digital") || g_str_has_prefix(m->name, "LVDS") )
-        return _( LVDS ? "External DVI Monitor" : "DVI Monitor");
+        return LVDS ? _("External DVI Monitor") : _("DVI Monitor");
     else if( g_str_has_prefix( m->name, "TV" ) || g_str_has_prefix(m->name, "S-Video") )
         return _("TV");
     else if( strcmp( m->name, "default" ) == 0 )
@@ -100,22 +118,41 @@ static gboolean get_xrandr_info()
         return FALSE;
     }
 
-    regex = g_regex_new( "([a-zA-Z]+[-0-9]*) +connected .*((\n +[0-9]+x[0-9]+[^\n]+)+)",
+    regex = g_regex_new( "\n([-a-zA-Z]+[-0-9]*) +connected ([^(\n ]*)[^\n]*"
+                         "((\n +[0-9]+x[0-9]+[^\n]+)+)",
                          0, 0, NULL );
     if( g_regex_match( regex, output, 0, &match ) )
     {
         do {
             Monitor* m = g_new0( Monitor, 1 );
-            char *modes = g_match_info_fetch( match, 2 );
+            char *modes = g_match_info_fetch( match, 3 );
+            char *coords = g_match_info_fetch(match, 2);
             char **lines, **line;
-            int imode = 0;
+            char *ptr;
+            int imode = 0, x = -1, y = -1;
 
             m->active_mode = m->active_rate = -1;
             m->pref_mode = m->pref_rate = -1;
             m->name = g_match_info_fetch( match, 1 );
+            ptr = strchr(coords, '+');
+            if (ptr != NULL)
+            {
+                ptr++;
+                x = strtol(ptr, &ptr, 10);
+                if (*ptr++ == '+')
+                    y = strtol(ptr, &ptr, 10);
+            }
+            /* g_debug("name '%s' coords '%s'=>%d:%d modes%.20s...",m->name,coords,x,y,&modes[1]); */
+            if (x < 0 || y < 0 || (x == 0 && y == 0))
+                m->placement = PLACEMENT_DEFAULT;
+            else if (x == 0)
+                m->placement = PLACEMENT_BELOW;
+            else if (y == 0)
+                m->placement = PLACEMENT_RIGHT;
 
             // check if this is the built-in LCD of laptop
-            if( ! LVDS && (strcmp( m->name, "LVDS" ) == 0 || strcmp( m->name, "PANEL" ) == 0) )
+            if (! LVDS && (g_str_has_prefix(m->name, "LVDS") ||
+                           g_str_has_prefix(m->name, "PANEL")))
                 LVDS = m;
 
             lines = g_strsplit( modes, "\n", -1 );
@@ -128,7 +165,7 @@ static gboolean get_xrandr_info()
                     continue;
                 strv = g_ptr_array_sized_new(8);
                 g_ptr_array_add( strv, g_strdup(str) );
-                while( str = strtok( NULL, " ") )
+                while ((str = strtok( NULL, " ")))
                 {
                     if( *str )
                     {
@@ -141,12 +178,12 @@ static gboolean get_xrandr_info()
                         else
                             g_ptr_array_add( strv, str );
 
-                        if( star = strchr( str, '*' ) )
+                        if ((star = strchr( str, '*' )))
                         {
                             m->active_mode = imode;
                             m->active_rate = irate;
                         }
-                        if( plus = strchr( str, '+' ) )
+                        if ((plus = strchr( str, '+' )))
                         {
                             m->pref_mode = imode;
                             m->pref_rate = irate;
@@ -164,6 +201,7 @@ static gboolean get_xrandr_info()
             }
             g_strfreev( lines );
             g_free( modes );
+            g_free(coords);
             monitors = g_slist_prepend( monitors, m );
         }while( g_match_info_next( match, NULL ) );
 
@@ -178,6 +216,27 @@ static gboolean get_xrandr_info()
     return TRUE;
 }
 
+static void on_enable_toggled(GtkToggleButton *tb, Monitor* m)
+{
+    GSList *l;
+    Monitor *fixed = LVDS ? LVDS : monitors->data;
+    int i;
+    gboolean can_position;
+
+    for (l = monitors, i = 0; l; l = l->next)
+        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m->enable)))
+            i++;
+    can_position = (i > 1);
+
+    if (monitors->next != NULL) for (l = monitors; l; l = l->next)
+    {
+        Monitor *m = (Monitor*)l->data;
+        gtk_widget_set_sensitive(GTK_WIDGET(m->pos_combo), can_position && (m != fixed));
+        if (!can_position || m == fixed)
+            gtk_combo_box_set_active(GTK_COMBO_BOX(m->pos_combo), 0);
+    }
+}
+
 static void on_res_sel_changed( GtkComboBox* cb, Monitor* m )
 {
     char** rate;
@@ -234,7 +293,7 @@ static void on_about( GtkButton* btn, gpointer parent )
     gtk_about_dialog_set_version ( (GtkAboutDialog*)about_dlg, VERSION );
     gtk_about_dialog_set_program_name ( (GtkAboutDialog*)about_dlg, _( "LXRandR" ) );
     //gtk_about_dialog_set_logo( (GtkAboutDialog*)about_dlg, gdk_pixbuf_new_from_file(  PACKAGE_DATA_DIR"/pixmaps/lxrandr.png", NULL ) );
-    gtk_about_dialog_set_copyright ( (GtkAboutDialog*)about_dlg, _( "Copyright (C) 2008-2011" ) );
+    gtk_about_dialog_set_copyright ( (GtkAboutDialog*)about_dlg, _( "Copyright (C) 2008-2014" ) );
     gtk_about_dialog_set_comments ( (GtkAboutDialog*)about_dlg, _( "Monitor configuration tool for LXDE" ) );
     gtk_about_dialog_set_license ( (GtkAboutDialog*)about_dlg, "This program is free software; you can redistribute it and/or\nmodify it under the terms of the GNU General Public License\nas published by the Free Software Foundation; either version 2\nof the License, or (at your option) any later version.\n\nmw program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License\nalong with mw program; if not, write to the Free Software\nFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA." );
     gtk_about_dialog_set_website ( (GtkAboutDialog*)about_dlg, "http://lxde.org/" );
@@ -247,62 +306,135 @@ static void on_about( GtkButton* btn, gpointer parent )
 }
 
 
+static void prepare_try_values_from_GUI()
+{
+    GSList *l;
+
+    for (l = monitors; l; l = l->next)
+    {
+        Monitor *m = (Monitor*)l->data;
+        if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m->enable)))
+            m->try_mode = -1;
+        else
+#if GTK_CHECK_VERSION(2, 24, 0)
+            m->try_mode = gtk_combo_box_get_active(GTK_COMBO_BOX(m->res_combo));
+        m->try_rate = gtk_combo_box_get_active(GTK_COMBO_BOX(m->rate_combo));
+        if (m->pos_combo)
+            m->try_placement = gtk_combo_box_get_active(GTK_COMBO_BOX(m->pos_combo));
+#else
+            m->try_mode = gtk_combo_box_get_active(m->res_combo);
+        m->try_rate = gtk_combo_box_get_active(m->rate_combo);
+        if (m->pos_combo)
+            m->try_placement = gtk_combo_box_get_active(m->pos_combo);
+#endif
+        /* g_debug("mode %d rate %d placement %d",m->try_mode,m->try_rate,m->try_placement); */
+    }
+}
+
 static GString* get_command_xrandr_info()
 {
 
     GSList* l;
     GString *cmd = g_string_sized_new( 1024 );
 
-    g_string_assign( cmd, "xrandr" );
+    g_string_assign( cmd, "sh -c 'xrandr" );
 
     for( l = monitors; l; l = l->next )
     {
         Monitor* m = (Monitor*)l->data;
         g_string_append( cmd, " --output " );
         g_string_append( cmd, m->name );
-        g_string_append_c( cmd, ' ' );
 
         // if the monitor is turned on
-        if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(m->enable) ) )
+        if (m->try_mode >= 0)
         {
-#if GTK_CHECK_VERSION(2, 24, 0)
-            int sel_res = gtk_combo_box_get_active( GTK_COMBO_BOX(m->res_combo) );
-            int sel_rate = gtk_combo_box_get_active( GTK_COMBO_BOX(m->rate_combo) );
-#else
-            int sel_res = gtk_combo_box_get_active( m->res_combo );
-            int sel_rate = gtk_combo_box_get_active( m->rate_combo );
-#endif
-
-            if( sel_res < 1 ) // auto resolution
+            if( m->try_mode < 1 ) // auto resolution
             {
-                g_string_append( cmd, "--auto" );
+                g_string_append( cmd, " --auto" );
             }
             else
             {
-                g_string_append( cmd, "--mode " );
-                ++sel_res;  // the fist item in the combo box is "Auto", indecis of resolutions are 1, 2, 3...
-#if GTK_CHECK_VERSION(2, 24, 0)
-                g_string_append( cmd, gtk_combo_box_text_get_active_text(m->res_combo) );
-#else
-                g_string_append( cmd, gtk_combo_box_get_active_text(m->res_combo) );
-#endif
-                if( sel_rate >= 1 ) // not auto refresh rate
+                GtkTreeModel *model;
+                GtkTreePath *path;
+                gchar *text;
+                GtkTreeIter iter;
+                gint text_column;
+
+                g_string_append( cmd, " --mode " );
+                model = gtk_combo_box_get_model(GTK_COMBO_BOX(m->res_combo));
+                text_column = gtk_combo_box_get_entry_text_column(GTK_COMBO_BOX(m->res_combo));
+                path = gtk_tree_path_new_from_indices(m->try_mode, -1);
+                gtk_tree_model_get_iter(model, &iter, path);
+                gtk_tree_model_get(model, &iter, text_column, &text, -1);
+                gtk_tree_path_free(path);
+                g_string_append(cmd, text);
+                g_free(text);
+                if( m->try_rate >= 1 ) // not auto refresh rate
                 {
                     g_string_append( cmd, " --rate " );
-#if GTK_CHECK_VERSION(2, 24, 0)
-                    g_string_append( cmd, gtk_combo_box_text_get_active_text(m->rate_combo) );
-#else
-                    g_string_append( cmd, gtk_combo_box_get_active_text(m->rate_combo) );
-#endif
+                    model = gtk_combo_box_get_model(GTK_COMBO_BOX(m->rate_combo));
+                    text_column = gtk_combo_box_get_entry_text_column(GTK_COMBO_BOX(m->rate_combo));
+                    path = gtk_tree_path_new_from_indices(m->try_rate, -1);
+                    gtk_tree_model_get_iter(model, &iter, path);
+                    gtk_tree_model_get(model, &iter, text_column, &text, -1);
+                    gtk_tree_path_free(path);
+                    g_string_append(cmd, text);
+                    g_free(text);
                 }
             }
+            if (m == LVDS) ; /* it's fixed, no positioning */
+            else if (LVDS != NULL)
+            {
+                switch (m->try_placement)
+                {
+                case PLACEMENT_RIGHT:
+                    g_string_append(cmd, " --right-of ");
+                    break;
+                case PLACEMENT_ABOVE:
+                    g_string_append(cmd, " --above ");
+                    break;
+                case PLACEMENT_LEFT:
+                    g_string_append(cmd, " --left-of ");
+                    break;
+                case PLACEMENT_BELOW:
+                    g_string_append(cmd, " --below ");
+                    break;
+                case PLACEMENT_DEFAULT:
+                    g_string_append(cmd, " --same-as ");
+                }
+                g_string_append(cmd, LVDS->name);
+            }
+            else if (l != monitors)
+            {
+                Monitor *first = (Monitor*)monitors->data;
+                /* not notebook */
+                switch (m->try_placement)
+                {
+                case PLACEMENT_RIGHT:
+                    g_string_append(cmd, " --right-of ");
+                    break;
+                case PLACEMENT_ABOVE:
+                    g_string_append(cmd, " --above ");
+                    break;
+                case PLACEMENT_LEFT:
+                    g_string_append(cmd, " --left-of ");
+                    break;
+                case PLACEMENT_BELOW:
+                    g_string_append(cmd, " --below ");
+                    break;
+                case PLACEMENT_DEFAULT:
+                    g_string_append(cmd, " --same-as ");
+                }
+                g_string_append(cmd, first->name);
+            }
 
-            g_string_append( cmd, "" );
+            /* g_string_append( cmd, "" ); */
 
         }
         else    // turn off
-            g_string_append( cmd, "--off" );
+            g_string_append( cmd, " --off" );
     }
+    g_string_append_c(cmd, '\'');
 
     return cmd;
 
@@ -318,7 +450,10 @@ static void save_configuration()
     char* file, *data;
     gsize len;
 
-    GString *cmd = get_command_xrandr_info();
+    GString *cmd;
+
+    prepare_try_values_from_GUI();
+    cmd = get_command_xrandr_info();
 
     /* create user autostart dir */
     dirname = g_build_filename(g_get_user_config_dir(), "autostart", NULL);
@@ -343,16 +478,76 @@ static void save_configuration()
     g_file_set_contents(file, data, len, NULL);
     g_key_file_free (kf);
     g_free(file);
-    g_free(data);    
+    g_free(data);
+    g_string_free(cmd, TRUE);
+}
 
+static gboolean cancel_confirmation(gpointer data)
+{
+    if (!g_source_is_destroyed(g_main_current_source()))
+        gtk_dialog_response(data, GTK_RESPONSE_CANCEL);
+    return FALSE;
 }
 
 static void set_xrandr_info()
 {
+    GString *cmd;
+    GSList *l;
 
-    GString *cmd = get_command_xrandr_info();
+    prepare_try_values_from_GUI();
+    cmd = get_command_xrandr_info();
+    /* g_debug("trying: %s", cmd->str); */
 
-    g_spawn_command_line_sync( cmd->str, NULL, NULL, NULL, NULL );
+    if (g_spawn_command_line_sync( cmd->str, NULL, NULL, NULL, NULL ))
+    {
+        /* open a dialog box and wait 15 seconds */
+        GtkWidget *confirmation;
+        guint timer;
+        int responce;
+
+        confirmation = gtk_message_dialog_new(GTK_WINDOW(dlg), GTK_DIALOG_MODAL,
+                                              GTK_MESSAGE_QUESTION,
+                                              GTK_BUTTONS_NONE,
+                                              _("Is everything OK? Confirm within 15 seconds,"
+                                                " otherwise previous state will be restored."));
+        gtk_dialog_add_buttons(GTK_DIALOG(confirmation),
+                               _("_OK"), GTK_RESPONSE_ACCEPT,
+                               _("_Abort"), GTK_RESPONSE_CANCEL,
+                               NULL);
+        gtk_dialog_set_default_response(GTK_DIALOG(confirmation), GTK_RESPONSE_CANCEL);
+        timer = gdk_threads_add_timeout(15000, cancel_confirmation, confirmation);
+        responce = gtk_dialog_run(GTK_DIALOG(confirmation));
+        g_source_remove(timer);
+        gtk_widget_destroy(confirmation);
+        /* if not confirmed then set GUI to fallback values, reset xrandr,
+           then restore all GUI again */
+        if (responce == GTK_RESPONSE_ACCEPT)
+        {
+            for (l = monitors; l; l = l->next)
+            {
+                Monitor *m = (Monitor*)l->data;
+
+                m->active_mode = m->try_mode;
+                m->active_rate = m->try_rate;
+                m->placement = m->try_placement;
+            }
+        }
+        else
+        {
+            for (l = monitors; l; l = l->next)
+            {
+                Monitor *m = (Monitor*)l->data;
+
+                m->try_mode = m->active_mode;
+                m->try_rate = m->active_rate;
+                m->try_placement = m->placement;
+            }
+            g_string_free(cmd, TRUE);
+            cmd = get_command_xrandr_info();
+            /* g_debug("recovering: %s", cmd->str); */
+            g_spawn_command_line_sync(cmd->str, NULL, NULL, NULL, NULL);
+        }
+    }
 
     g_string_free( cmd, TRUE );
 }
@@ -362,9 +557,13 @@ static void choose_max_resolution( Monitor* m )
 #if GTK_CHECK_VERSION(2, 24, 0)
     if( gtk_tree_model_iter_n_children( gtk_combo_box_get_model(GTK_COMBO_BOX(m->res_combo)), NULL ) > 1 )
         gtk_combo_box_set_active( GTK_COMBO_BOX(m->res_combo), 1 );
+    if (m->pos_combo)
+        gtk_combo_box_set_active(GTK_COMBO_BOX(m->pos_combo), PLACEMENT_DEFAULT);
 #else
     if( gtk_tree_model_iter_n_children( gtk_combo_box_get_model(m->res_combo), NULL ) > 1 )
         gtk_combo_box_set_active( m->res_combo, 1 );
+    if (m->pos_combo)
+        gtk_combo_box_set_active(m->pos_combo, PLACEMENT_DEFAULT);
 #endif
 }
 
@@ -398,10 +597,30 @@ static void on_quick_option( GtkButton* btn, gpointer data )
             gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(m->enable), m == LVDS );
         }
         break;
+    case 4: // external right of LVDS
+        for( l = monitors; l; l = l->next )
+        {
+            Monitor* m = (Monitor*)l->data;
+            choose_max_resolution( m );
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m->enable), TRUE);
+            if (m != LVDS)
+                gtk_combo_box_set_active(GTK_COMBO_BOX(m->pos_combo), PLACEMENT_RIGHT);
+        }
+        break;
+    case 5: // external above of LVDS
+        for( l = monitors; l; l = l->next )
+        {
+            Monitor* m = (Monitor*)l->data;
+            choose_max_resolution( m );
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m->enable), TRUE);
+            if (m != LVDS)
+                gtk_combo_box_set_active(GTK_COMBO_BOX(m->pos_combo), PLACEMENT_ABOVE);
+        }
+        break;
     default:
         return;
     }
-    gtk_dialog_response( GTK_DIALOG(dlg), GTK_RESPONSE_OK );
+//    gtk_dialog_response( GTK_DIALOG(dlg), GTK_RESPONSE_OK );
 //    set_xrandr_info();
 }
 
@@ -415,13 +634,20 @@ static void on_response( GtkDialog* dialog, int response, gpointer user_data )
         {
             Monitor* m = (Monitor*)l->data;
             if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(m->enable) ) )
-                return;
+                break;
+        }
+        if (l != NULL)
+            set_xrandr_info();
+        else
+        {
+            msg = gtk_message_dialog_new( GTK_WINDOW(dialog), 0, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                                         _("You cannot turn off all monitors. "
+                                           "Otherwise, you will not be able to "
+                                           "turn them on again since this tool "
+                                           "is not accessible without monitor.") );
+            gtk_dialog_run( GTK_DIALOG(msg) );
+            gtk_widget_destroy( msg );
         }
-
-        msg = gtk_message_dialog_new( GTK_WINDOW(dialog), 0, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
-                                     _("You cannot turn off all monitors. Otherwise, you will not be able to turn them on again since this tool is not accessable without monitor.") );
-        gtk_dialog_run( GTK_DIALOG(msg) );
-        gtk_widget_destroy( msg );
 
         // block the response
         g_signal_stop_emission_by_name( dialog, "response" );
@@ -446,11 +672,14 @@ int main(int argc, char** argv)
 {
     GtkWidget *notebook, *vbox, *frame, *label, *hbox, *check, *btn;
     GSList* l;
+    Monitor *fixed;
+    int i;
+    gboolean can_position;
 
 #ifdef ENABLE_NLS
-	bindtextdomain ( GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR );
-	bind_textdomain_codeset ( GETTEXT_PACKAGE, "UTF-8" );
-	textdomain ( GETTEXT_PACKAGE );
+    bindtextdomain ( GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR );
+    bind_textdomain_codeset ( GETTEXT_PACKAGE, "UTF-8" );
+    textdomain ( GETTEXT_PACKAGE );
 #endif
 
     gtk_init( &argc, &argv );
@@ -502,18 +731,30 @@ int main(int argc, char** argv)
         vbox = gtk_vbox_new( FALSE, 4 );
         gtk_container_set_border_width( GTK_CONTAINER(vbox), 8 );
 
-        btn = gtk_button_new_with_label( _("Show the same screen on both laptop LCD and external monitor") );
+        btn = gtk_radio_button_new_with_label(NULL, _("Show the same screen on both laptop LCD and external monitor"));
         g_signal_connect( btn, "clicked", G_CALLBACK(on_quick_option), GINT_TO_POINTER(1) );
         gtk_box_pack_start( GTK_BOX(vbox), btn, FALSE, TRUE , 4);
 
-        btn = gtk_button_new_with_label( _("Turn off laptop LCD and use external monitor only") );
+        btn = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(btn),
+                        _("Turn off laptop LCD and use external monitor only"));
         g_signal_connect( btn, "clicked", G_CALLBACK(on_quick_option), GINT_TO_POINTER(2) );
         gtk_box_pack_start( GTK_BOX(vbox), btn, FALSE, TRUE , 4);
 
-        btn = gtk_button_new_with_label( _("Turn off external monitor and use laptop LCD only") );
+        btn = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(btn),
+                        _("Turn off external monitor and use laptop LCD only"));
         g_signal_connect( btn, "clicked", G_CALLBACK(on_quick_option), GINT_TO_POINTER(3) );
         gtk_box_pack_start( GTK_BOX(vbox), btn, FALSE, TRUE , 4);
 
+        btn = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(btn),
+                        _("Place external monitor to the right of laptop LCD"));
+        g_signal_connect( btn, "clicked", G_CALLBACK(on_quick_option), GINT_TO_POINTER(4) );
+        gtk_box_pack_start( GTK_BOX(vbox), btn, FALSE, TRUE , 4);
+
+        btn = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(btn),
+                        _("Place external monitor above of laptop LCD"));
+        g_signal_connect( btn, "clicked", G_CALLBACK(on_quick_option), GINT_TO_POINTER(5) );
+        gtk_box_pack_start( GTK_BOX(vbox), btn, FALSE, TRUE , 4);
+
         gtk_notebook_append_page( GTK_NOTEBOOK(notebook), vbox, gtk_label_new( _("Quick Options") ) );
     }
     else
@@ -532,7 +773,41 @@ int main(int argc, char** argv)
                                     g_slist_length(monitors) ) );
     gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, TRUE, 2 );
 
-    int i;
+    for (l = monitors, i = 0; l; l = l->next)
+        if (((Monitor*)l->data)->active_mode >= 0)
+            i++;
+    can_position = (i > 1);
+
+    /* correct placements */
+    fixed = LVDS ? LVDS : monitors->data;
+    if (fixed->placement != PLACEMENT_DEFAULT)
+    {
+        for (l = monitors, i = 0; l; l = l->next)
+        {
+            Monitor *m = (Monitor*)l->data;
+
+            if (m->placement != PLACEMENT_DEFAULT)
+                continue; /* FIXME: how to handle corners? */
+            switch (fixed->placement)
+            {
+            case PLACEMENT_LEFT:
+                m->placement = PLACEMENT_RIGHT;
+                break;
+            case PLACEMENT_RIGHT:
+                m->placement = PLACEMENT_LEFT;
+                break;
+            case PLACEMENT_ABOVE:
+                m->placement = PLACEMENT_BELOW;
+                break;
+            case PLACEMENT_BELOW:
+                m->placement = PLACEMENT_ABOVE;
+                break;
+            case PLACEMENT_DEFAULT: ;
+            }
+        }
+        fixed->placement = PLACEMENT_DEFAULT;
+    }
+
     for( l = monitors, i = 0; l; l = l->next, ++i )
     {
         Monitor* m = (Monitor*)l->data;
@@ -547,14 +822,41 @@ int main(int argc, char** argv)
 
         check = gtk_check_button_new_with_label( _("Turn On") );
         m->enable = GTK_CHECK_BUTTON(check);
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), m->active_mode >= 0);
 
         // turn off screen is not allowed since there should be at least one monitor available.
         if( g_slist_length( monitors ) == 1 )
             gtk_widget_set_sensitive( GTK_WIDGET(m->enable), FALSE );
+        else
+            g_signal_connect(m->enable, "toggled", G_CALLBACK(on_enable_toggled), m);
 
         gtk_box_pack_start( GTK_BOX(hbox), check, FALSE, TRUE, 6 );
-        if( m->active_mode >= 0 )
-            gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(check), TRUE );
+
+        if (monitors->next != NULL) /* g_slist_length(monitors) > 0 */
+        {
+            label = gtk_label_new(_("Position:"));
+            gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 2);
+#if GTK_CHECK_VERSION(2, 24, 0)
+            m->pos_combo = GTK_COMBO_BOX_TEXT(gtk_combo_box_text_new());
+            gtk_combo_box_text_append_text(m->pos_combo, _("Default"));
+            gtk_combo_box_text_append_text(m->pos_combo, _("On right"));
+            gtk_combo_box_text_append_text(m->pos_combo, _("Above"));
+            gtk_combo_box_text_append_text(m->pos_combo, _("On left"));
+            gtk_combo_box_text_append_text(m->pos_combo, _("Below"));
+            gtk_combo_box_set_active(GTK_COMBO_BOX(m->pos_combo), m->placement);
+#else
+            m->pos_combo = GTK_COMBO_BOX(gtk_combo_box_new_text());
+            gtk_combo_box_append_text(m->res_combo, _("Default"));
+            gtk_combo_box_append_text(m->res_combo, _("On right"));
+            gtk_combo_box_append_text(m->res_combo, _("Above"));
+            gtk_combo_box_append_text(m->pos_combo, _("On left"));
+            gtk_combo_box_append_text(m->pos_combo, _("Below"));
+            gtk_combo_box_set_active(m->pos_combo, m->placement);
+#endif
+            if (m == fixed || !can_position || m->active_mode < 0)
+                gtk_widget_set_sensitive(GTK_WIDGET(m->pos_combo), FALSE);
+            gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(m->pos_combo), FALSE, TRUE, 2);
+        }
 
         label = gtk_label_new( _("Resolution:") );
         gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, TRUE, 2 );
@@ -592,21 +894,23 @@ int main(int argc, char** argv)
 #endif
         }
 
+        m->active_rate++;
 #if GTK_CHECK_VERSION(2, 24, 0)
         gtk_combo_box_set_active( GTK_COMBO_BOX(m->res_combo), m->active_mode + 1 );
-        gtk_combo_box_set_active( GTK_COMBO_BOX(m->rate_combo), m->active_rate + 1 );
+        gtk_combo_box_set_active( GTK_COMBO_BOX(m->rate_combo), m->active_rate );
 #else
         gtk_combo_box_set_active( m->res_combo, m->active_mode + 1 );
-        gtk_combo_box_set_active( m->rate_combo, m->active_rate + 1 );
+        gtk_combo_box_set_active( m->rate_combo, m->active_rate );
 #endif
+        if (m->active_mode >= 0)
+            m->active_mode++; /* let it stay -1 for inactive button */
     }
 
     gtk_widget_show_all( dlg );
 
-    if( gtk_dialog_run( (GtkDialog*)dlg ) == GTK_RESPONSE_OK )
-        set_xrandr_info();
+    gtk_dialog_run((GtkDialog*)dlg);
 
     gtk_widget_destroy( dlg );
 
-	return 0;
+    return 0;
 }
